CREATE OR REPLACE PACKAGE SUP_COTACAO AS
    /*
     * Teknisa Service
     * Package com procedimentos das rotinas relacionadas a cotação de preços
     */

    /*Constants*/
    K_S                     CONSTANT VARCHAR2(01) := 'S';
    K_N                     CONSTANT VARCHAR2(01) := 'N';
    K_T                     CONSTANT VARCHAR2(01) := 'T';
    K_N1                    CONSTANT NUMBER       := 1;
    K_N0                    CONSTANT NUMBER       := 0;
    K_SUCCESS               CONSTANT VARCHAR2(07) := 'SUCCESS';
    K_ERROR                 CONSTANT VARCHAR2(05) := 'ERROR';
    K_PZOPGTPRFO            CONSTANT VARCHAR2(30) := 'PZOPGTPRFO';
    K_PRAZOPGTFORN          CONSTANT VARCHAR2(30) := 'PRAZOPGTFORN';
    K_COTACAO_CONCLUIDA     CONSTANT VARCHAR2(30) := 'COTACAO_CONCLUIDA_IDCOTCONCLUI';
    K_DSMET_DUPLICA_COTACAO CONSTANT VARCHAR2(30) := 'SUP_COTACAO.DUPLICA_COTACAO';

    /*Subtype*/
    SUBTYPE S_RECORD_COLETAPR     IS COLETAPR%ROWTYPE;
    SUBTYPE S_RECORD_ITEMCOLE     IS ITEMCOLE%ROWTYPE;
    SUBTYPE S_RECORD_LOGCOLETAPR  IS LOGCOLETAPR%ROWTYPE;
    SUBTYPE S_RECORD_LOGITEMCOLE  IS LOGITEMCOLE%ROWTYPE;
    SUBTYPE S_RECORD_PZOPGTITMCOL IS PZOPGTITMCOL%ROWTYPE;

    PROCEDURE INSERT_COLETAPR(P_REGISTRO      IN OUT S_RECORD_COLETAPR,
                              P_CDFORNECED    IN FORNECEDOR.CDFORNECED%TYPE,
                              P_PARASUPRI     IN PARASUPRI%ROWTYPE,
                              P_FORNECEDORES OUT FORNECEDOR%ROWTYPE,
                              P_ERROR        OUT VARCHAR2);

    PROCEDURE INSERT_ITEMCOLE(P_REGISTRO     IN OUT S_RECORD_ITEMCOLE,
                              P_PARASUPRI    IN PARASUPRI%ROWTYPE,
                              P_FORNECEDORES IN FORNECEDOR%ROWTYPE,
                              P_ERROR       OUT VARCHAR2);

    PROCEDURE DELETE_PZOPGTITMCOL(P_REGISTRO IN S_RECORD_ITEMCOLE,
                                  P_ERROR   OUT VARCHAR2);

    PROCEDURE INSERT_PZOPGTITMCOL(P_REGISTRO   IN S_RECORD_ITEMCOLE,
                                  P_CDPRODFORN IN PRODFORN.CDPRODUTO%TYPE DEFAULT NULL,
                                  P_ERROR     OUT VARCHAR2);

    PROCEDURE INSERT_LOGCOLETAPR(P_COLETAPR      IN S_RECORD_COLETAPR,
                                 P_DSENDUSULOG   IN LOGCOLETAPR.DSENDUSULOG%TYPE,
                                 P_DSNMUSULOG    IN LOGCOLETAPR.DSNMUSULOG%TYPE,
                                 P_DSRDMUSULOG   IN LOGCOLETAPR.DSRDMUSULOG%TYPE,
                                 P_DSMETUSULOG   IN LOGCOLETAPR.DSMETUSULOG%TYPE,
                                 P_CDOPERINCLOG  IN LOGCOLETAPR.CDOPERINCLOG%TYPE,
                                 P_IDOPERCOTLOG  IN LOGCOLETAPR.IDOPERCOTLOG%TYPE,
                                 P_DSMESSAGERROR IN LOGCOLETAPR.DSMESSAGERROR%TYPE); 

    PROCEDURE INSERT_LOGITEMCOLE(P_ITEMCOLE      IN S_RECORD_ITEMCOLE,
                                 P_DSENDUSULIC   IN LOGITEMCOLE.DSENDUSULIC%TYPE,
                                 P_DSNMUSULIC    IN LOGITEMCOLE.DSNMUSULIC%TYPE,
                                 P_DSRDMUSULIC   IN LOGITEMCOLE.DSRDMUSULIC%TYPE,
                                 P_DSMETUSULIC   IN LOGITEMCOLE.DSMETUSULIC%TYPE,
                                 P_CDOPERLIC     IN LOGITEMCOLE.CDOPERLIC%TYPE,
                                 P_IDOPERALIC    IN LOGITEMCOLE.IDOPERALIC%TYPE,
                                 P_DSMESSAGERROR IN LOGITEMCOLE.DSMESSAGERROR%TYPE);                             




    PROCEDURE COTACAO_OPL(P_CDFILIAL    IN COLETAPR.CDFILIAL%TYPE,
                          P_NRCOLEORIG  IN COLETAPR.NRCOLEPRECO%TYPE,
                          P_NRCOLEPRECO IN COLETAPR.NRCOLEPRECO%TYPE,
                          P_FORNOL      IN VARCHAR2,
                          P_REAJUSTE    IN NUMBER,
                          P_DTNOVINIVAL IN COLETAPR.DTINIVALCOL%TYPE,
                          P_DTNOVFINVAL IN COLETAPR.DTFINVALCOL%TYPE,
                          P_CDOPERINCL  IN OPERADOR.CDOPERADOR%TYPE);
    
    PROCEDURE DUPLICA_COTACAO(P_DUPLICA_CORRENTE IN BOOLEAN,
                              P_CDFILIAL         IN COLETAPR.CDFILIAL%TYPE,
                              P_NRCOLEPRECO      IN COLETAPR.NRCOLEPRECO%TYPE,
                              P_CDCOMPRADOR      IN VARCHAR2                  DEFAULT NULL,
                              P_TCDCOMPRADOR     IN VARCHAR2                  DEFAULT NULL,
                              P_CDOPERADOR       IN OPERADOR.CDOPERADOR%TYPE  DEFAULT NULL,
                              P_DTINIVALCOL      IN COLETAPR.DTINIVALCOL%TYPE DEFAULT NULL,
                              P_DTFINVALCOL      IN COLETAPR.DTFINVALCOL%TYPE DEFAULT NULL,
                              P_CDREGIAO         IN VARCHAR2                  DEFAULT NULL,
                              P_DTNOVINIVAL      IN COLETAPR.DTINIVALCOL%TYPE,
                              P_DTNOVFINVAL      IN COLETAPR.DTFINVALCOL%TYPE,
                              P_DTCOLEPREC       IN COLETAPR.DTCOLEPREC%TYPE,
                              P_CDOPERINCL       IN OPERADOR.CDOPERADOR%TYPE,
                              P_REAJUSTE         IN NUMBER,
                              P_CDFORNOL         IN VARCHAR2                  DEFAULT NULL,
                              P_MESSAGE         OUT VARCHAR2);
END SUP_COTACAO;
/

CREATE OR REPLACE PACKAGE BODY SUP_COTACAO AS

    PROCEDURE INSERT_COLETAPR(P_REGISTRO      IN OUT S_RECORD_COLETAPR,
                              P_CDFORNECED    IN FORNECEDOR.CDFORNECED%TYPE,
                              P_PARASUPRI     IN PARASUPRI%ROWTYPE,
                              P_FORNECEDORES OUT FORNECEDOR%ROWTYPE,
                              P_ERROR        OUT VARCHAR2) IS
        C_FORNECEDORES FORNECEDOR%ROWTYPE;                                           
    BEGIN        
        SUP_VALIDATIONS.CHECK_FORNECEDOR(P_CDFORNECED   => P_CDFORNECED,
                                         P_DTPEDIDO     => P_REGISTRO.DTFINVALCOL,
                                         P_PARASUPRI    => P_PARASUPRI,
                                         P_FORNECEDORES => C_FORNECEDORES,
                                         P_MESSAGE      => P_ERROR);
        IF P_ERROR IS NULL THEN   
            
            P_REGISTRO.IDSITUCOLETA := CASE NVL(P_PARASUPRI.IDUTISTATCOT, 'N') WHEN 'S' THEN 'A' ELSE 'C' END;
            P_REGISTRO.DTINCLCOLET  := SYSDATE;  

            SELECT TO_CHAR(TO_DATE(P_REGISTRO.DTINIVALCOL, 'DD/MM/YYYY'),'WW'), 
                   TO_CHAR(TO_DATE(P_REGISTRO.DTFINVALCOL, 'DD/MM/YYYY'),'WW')
              INTO P_REGISTRO.NRSEMAINIC, 
                   P_REGISTRO.NRSEMAFINAL
              FROM DUAL;
            
            BEGIN
                INSERT INTO COLETAPR
                     VALUES P_REGISTRO;
            EXCEPTION
                WHEN OTHERS THEN
                    P_ERROR := SQLERRM;
            END;

            P_FORNECEDORES := NULL;
        ELSE
            P_FORNECEDORES := C_FORNECEDORES;
        END IF;
    END INSERT_COLETAPR;

    PROCEDURE INSERT_ITEMCOLE(P_REGISTRO     IN OUT S_RECORD_ITEMCOLE,
                              P_PARASUPRI    IN PARASUPRI%ROWTYPE,
                              P_FORNECEDORES IN FORNECEDOR%ROWTYPE,
                              P_ERROR       OUT VARCHAR2) IS
        V_PRODFORN PRODFORN%ROWTYPE;
        V_ERROR    VARCHAR2(4000);
    BEGIN
        /*Fornecedor atende região*/
        V_ERROR := SUP_VALIDATIONS.FORNECEDOR_ATENDE_REGIAO(P_CDFORNECED => P_REGISTRO.CDFORNECED,
                                                            P_CDREGIAO   => P_REGISTRO.CDREGIAO);
        IF V_ERROR IS NULL THEN
            /*Qualificação produto por fornecedor*/
            V_ERROR := SUP_VALIDATIONS.QUALIFICACAO_PROD_FORN(P_CDPRODUTO  => P_REGISTRO.CDPRODUTO,
                                                              P_CDFORNECED => P_REGISTRO.CDFORNECED,
                                                              P_DTPEDIDO   => P_REGISTRO.DTFINVALITM,
                                                              P_PARASUPRI  => P_PARASUPRI);
        END IF;

        IF V_ERROR IS NULL THEN
            /*Qualificação marca por produto*/
            V_ERROR := SUP_VALIDATIONS.QUALIFICACAO_MARCA_PRODUTO(P_CDPRODUTO => P_REGISTRO.CDPRODUTO,
                                                                  P_CDMARCA   => P_REGISTRO.CDMARCA,
                                                                  P_DTPEDIDO  => P_REGISTRO.DTFINVALITM,
                                                                  P_PARASUPRI => P_PARASUPRI);

        END IF;

        IF V_ERROR IS NULL THEN
            /*Produto não cadastrado para o fornecedor*/
            V_ERROR := SUP_VALIDATIONS.CHECK_PRODUTO_FORNECEDOR(P_CDPRODUTO  => P_REGISTRO.CDPRODUTO,
                                                                P_CDFORNECED => P_REGISTRO.CDFORNECED);
        END IF;

        IF V_ERROR IS NULL THEN
            BEGIN
                SELECT *
                  INTO V_PRODFORN
                  FROM PRODFORN
                 WHERE CDFORNECED = P_REGISTRO.CDFORNECED
                   AND EXISTS (SELECT 1
                                 FROM PRODFORN PF
                                 JOIN PRODUTO PI ON PI.CDPRODUTO = PF.CDPRODUTO
                                 JOIN PRODUTO PR ON PR.CDARVPROD LIKE PI.CDARVPROD || '%'
                                WHERE PF.CDFORNECED = PRODFORN.CDFORNECED
                                  AND PI.CDPRODUTO  = PRODFORN.CDPRODUTO
                                  AND PR.CDPRODUTO  = P_REGISTRO.CDPRODUTO);
                
                P_REGISTRO.CDFORMPGTO   := V_PRODFORN.CDFORMPGTO; 
                P_REGISTRO.IDTPFOPGTCOL := V_PRODFORN.IDTPFORPGPRF;
                P_REGISTRO.NRPARCPGTCOL := V_PRODFORN.NRPARCPGTPRF;
                P_REGISTRO.NRDIAPGFXCOL := V_PRODFORN.NRDIAPGFIPRF;
                P_REGISTRO.IDUTDIAPGCOL := V_PRODFORN.IDUTDIAPGPRF;
                
                BEGIN
                    INSERT INTO ITEMCOLE
                        VALUES P_REGISTRO;
                EXCEPTION
                    WHEN OTHERS THEN
                        V_ERROR := SQLERRM;
                END;

                IF V_ERROR IS NULL THEN
                    DELETE_PZOPGTITMCOL(P_REGISTRO => P_REGISTRO,
                                        P_ERROR    => V_ERROR);

                    INSERT_PZOPGTITMCOL(P_REGISTRO   => P_REGISTRO,
                                        P_CDPRODFORN => V_PRODFORN.CDPRODUTO,
                                        P_ERROR      => V_ERROR);
                END IF;
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    P_REGISTRO.CDFORMPGTO   := P_FORNECEDORES.CDFORMPGTO; 
                    P_REGISTRO.IDTPFOPGTCOL := P_FORNECEDORES.IDTPFORPGTFO;
                    P_REGISTRO.NRPARCPGTCOL := P_FORNECEDORES.NRPARCPGTOFO;
                    P_REGISTRO.NRDIAPGFXCOL := P_FORNECEDORES.NRDIAPGFIXFO;
                    P_REGISTRO.IDUTDIAPGCOL := P_FORNECEDORES.IDUTDIAPGTFO;

                    BEGIN
                        INSERT INTO ITEMCOLE
                             VALUES P_REGISTRO;
                    EXCEPTION
                        WHEN OTHERS THEN
                            V_ERROR := SQLERRM;
                    END;

                    IF V_ERROR IS NULL THEN
                        DELETE_PZOPGTITMCOL(P_REGISTRO => P_REGISTRO,
                                            P_ERROR    => V_ERROR);

                        INSERT_PZOPGTITMCOL(P_REGISTRO => P_REGISTRO,
                                            P_ERROR    => V_ERROR);
                    END IF;
            END;
        END IF;

        P_ERROR := V_ERROR;
    END INSERT_ITEMCOLE;

    PROCEDURE DELETE_PZOPGTITMCOL(P_REGISTRO IN S_RECORD_ITEMCOLE,
                                  P_ERROR   OUT VARCHAR2) IS
    BEGIN
        DELETE PZOPGTITMCOL
         WHERE NRCOLEPRECO  = P_REGISTRO.NRCOLEPRECO
           AND CDFORNECED   = P_REGISTRO.CDFORNECED
           AND CDFILIAL     = P_REGISTRO.CDFILIAL
           AND CDREGIAO     = P_REGISTRO.CDREGIAO
           AND CDPRODUTO    = P_REGISTRO.CDPRODUTO
           AND CDMARCA      = P_REGISTRO.CDMARCA;
    EXCEPTION
        WHEN OTHERS THEN
            P_ERROR := SQLERRM;
    END DELETE_PZOPGTITMCOL;

    PROCEDURE INSERT_PZOPGTITMCOL(P_REGISTRO   IN S_RECORD_ITEMCOLE,
                                  P_CDPRODFORN IN PRODFORN.CDPRODUTO%TYPE DEFAULT NULL,
                                  P_ERROR     OUT VARCHAR2) IS
        V_RECORD S_RECORD_PZOPGTITMCOL;
    BEGIN
        V_RECORD.CDFILIAL    := P_REGISTRO.CDFILIAL;
        V_RECORD.NRCOLEPRECO := P_REGISTRO.NRCOLEPRECO;
        V_RECORD.CDPRODUTO   := P_REGISTRO.CDPRODUTO;
        V_RECORD.CDMARCA     := P_REGISTRO.CDMARCA;
        V_RECORD.CDREGIAO    := P_REGISTRO.CDREGIAO;
        
        FOR C_PRAZO IN (SELECT CDFORNECED,
                               NRDIASPGPRFO NRDIASPGTCOL,
                               VRPERCPGPRFO VRPERCPGTCOL,
                               NRORG,
                               IDATIVO
                          FROM PZOPGTPRFO
                         WHERE CDPRODUTO  = P_CDPRODFORN
                           AND CDFORNECED = P_REGISTRO.CDFORNECED
                         UNION ALL   
                        SELECT CDFORNECED,
                               NRDIASPGTOFO NRDIASPGTCOL,
                               VRPERCPGTOFO VRPERCPGTCOL,
                               NRORG,
                               IDATIVO
                          FROM PRAZOPGTFORN
                         WHERE CDFORNECED = P_REGISTRO.CDFORNECED
                           AND NOT EXISTS (SELECT 1
                                             FROM PZOPGTPRFO PZ
                                            WHERE PZ.CDPRODUTO    = P_CDPRODFORN
                                              AND PZ.CDFORNECED   = PRAZOPGTFORN.CDFORNECED))
        LOOP
            V_RECORD.CDFORNECED   := C_PRAZO.CDFORNECED; 
            V_RECORD.NRDIASPGTCOL := C_PRAZO.NRDIASPGTCOL;
            V_RECORD.VRPERCPGTCOL := C_PRAZO.VRPERCPGTCOL;
            V_RECORD.NRORG        := C_PRAZO.NRORG;
            V_RECORD.IDATIVO      := C_PRAZO.IDATIVO;

            BEGIN
                INSERT INTO PZOPGTITMCOL
                     VALUES V_RECORD;
            EXCEPTION
                WHEN OTHERS THEN
                    P_ERROR := SQLERRM;
            END;
        END LOOP;
    END INSERT_PZOPGTITMCOL;

    PROCEDURE INSERT_LOGCOLETAPR(P_COLETAPR      IN S_RECORD_COLETAPR,
                                 P_DSENDUSULOG   IN LOGCOLETAPR.DSENDUSULOG%TYPE,
                                 P_DSNMUSULOG    IN LOGCOLETAPR.DSNMUSULOG%TYPE,
                                 P_DSRDMUSULOG   IN LOGCOLETAPR.DSRDMUSULOG%TYPE,
                                 P_DSMETUSULOG   IN LOGCOLETAPR.DSMETUSULOG%TYPE,
                                 P_CDOPERINCLOG  IN LOGCOLETAPR.CDOPERINCLOG%TYPE,
                                 P_IDOPERCOTLOG  IN LOGCOLETAPR.IDOPERCOTLOG%TYPE,
                                 P_DSMESSAGERROR IN LOGCOLETAPR.DSMESSAGERROR%TYPE) IS
        V_REC_LOGCOLETAPR S_RECORD_LOGCOLETAPR;
    BEGIN
        V_REC_LOGCOLETAPR.CDFILIALLOG   := P_COLETAPR.CDFILIAL; 
        V_REC_LOGCOLETAPR.NRCOLEPRLOG   := P_COLETAPR.NRCOLEPRECO; 
        V_REC_LOGCOLETAPR.CDCOMPLOG     := P_COLETAPR.CDOPERADOR; 
        V_REC_LOGCOLETAPR.DTCOLEPRLOG   := P_COLETAPR.DTCOLEPREC; 
        V_REC_LOGCOLETAPR.NRSEMAINLOG   := P_COLETAPR.NRSEMAINIC; 
        V_REC_LOGCOLETAPR.NRSEMAFILOG   := P_COLETAPR.NRSEMAFINAL; 
        V_REC_LOGCOLETAPR.DTINIVALLOG   := P_COLETAPR.DTINIVALCOL; 
        V_REC_LOGCOLETAPR.DTFINVALLOG   := P_COLETAPR.DTFINVALCOL; 
        V_REC_LOGCOLETAPR.CDOPECOLLOG   := P_COLETAPR.CDOPERINCL; 
        V_REC_LOGCOLETAPR.DTULTCOLLOG   := SYSDATE;                 
        V_REC_LOGCOLETAPR.IDOPERCOTLOG  := P_IDOPERCOTLOG; 
        V_REC_LOGCOLETAPR.NRCOLEORILOG  := P_COLETAPR.NRCOLEORIGCO; 
        V_REC_LOGCOLETAPR.DTINCLITMLOG  := SYSDATE;
        V_REC_LOGCOLETAPR.DSMESSAGERROR := P_DSMESSAGERROR;

        V_REC_LOGCOLETAPR.DSENDUSULOG   := P_DSENDUSULOG;
        V_REC_LOGCOLETAPR.DSNMUSULOG    := P_DSNMUSULOG;
        V_REC_LOGCOLETAPR.DSRDMUSULOG   := P_DSRDMUSULOG;
        V_REC_LOGCOLETAPR.DSMETUSULOG   := SUBSTR(K_DSMET_DUPLICA_COTACAO||P_DSMETUSULOG, 0, 250);
        V_REC_LOGCOLETAPR.CDOPERINCLOG  := P_CDOPERINCLOG;

        INSERT INTO LOGCOLETAPR
             VALUES V_REC_LOGCOLETAPR;

    END INSERT_LOGCOLETAPR;

    PROCEDURE INSERT_LOGITEMCOLE(P_ITEMCOLE      IN S_RECORD_ITEMCOLE,
                                 P_DSENDUSULIC   IN LOGITEMCOLE.DSENDUSULIC%TYPE,
                                 P_DSNMUSULIC    IN LOGITEMCOLE.DSNMUSULIC%TYPE,
                                 P_DSRDMUSULIC   IN LOGITEMCOLE.DSRDMUSULIC%TYPE,
                                 P_DSMETUSULIC   IN LOGITEMCOLE.DSMETUSULIC%TYPE,
                                 P_CDOPERLIC     IN LOGITEMCOLE.CDOPERLIC%TYPE,
                                 P_IDOPERALIC    IN LOGITEMCOLE.IDOPERALIC%TYPE,
                                 P_DSMESSAGERROR IN LOGITEMCOLE.DSMESSAGERROR%TYPE) IS
        V_REC_LOGITEMCOLE S_RECORD_LOGITEMCOLE;
    BEGIN
        V_REC_LOGITEMCOLE.CDFILIALLIC   := P_ITEMCOLE.CDFILIAL; 
        V_REC_LOGITEMCOLE.NRCOLEPRELIC  := P_ITEMCOLE.NRCOLEPRECO; 
        V_REC_LOGITEMCOLE.CDMARCALIC    := P_ITEMCOLE.CDMARCA;
        V_REC_LOGITEMCOLE.CDREGIAOLIC   := P_ITEMCOLE.CDREGIAO;
        V_REC_LOGITEMCOLE.CDFORNECELIC  := P_ITEMCOLE.CDFORNECED;
        V_REC_LOGITEMCOLE.CDPRODUTOLIC  := P_ITEMCOLE.CDPRODUTO;
        V_REC_LOGITEMCOLE.DTINIVALLIC   := P_ITEMCOLE.DTINIVALITM;
        V_REC_LOGITEMCOLE.DTFINVALLIC   := P_ITEMCOLE.DTFINVALITM;
        V_REC_LOGITEMCOLE.DTOPERACLIC   := SYSDATE; 
        V_REC_LOGITEMCOLE.IDOPERALIC    := P_IDOPERALIC;
        V_REC_LOGITEMCOLE.DSRDMUSULIC   := P_DSRDMUSULIC;
        V_REC_LOGITEMCOLE.DSENDUSULIC   := P_DSENDUSULIC;
        V_REC_LOGITEMCOLE.DSNMUSULIC    := P_DSNMUSULIC;
        V_REC_LOGITEMCOLE.DSMETUSULIC   := SUBSTR(K_DSMET_DUPLICA_COTACAO||P_DSMETUSULIC, 0, 250);
        V_REC_LOGITEMCOLE.CDOPERLIC     := P_CDOPERLIC;
        V_REC_LOGITEMCOLE.VRPRECANTLIC  := 0;
        V_REC_LOGITEMCOLE.VRPRECDEPLIC  := P_ITEMCOLE.VRPRECUNIITM;
        V_REC_LOGITEMCOLE.IDITMLISPLIC  := P_ITEMCOLE.IDITMLISPREC;
        V_REC_LOGITEMCOLE.NRCOLEORILIC  := P_ITEMCOLE.NRCOLEORIGIC;
        V_REC_LOGITEMCOLE.DTINCLITMLIC  := SYSDATE;
        V_REC_LOGITEMCOLE.CDOPERINCLIC  := P_ITEMCOLE.CDOPERINCL;
        V_REC_LOGITEMCOLE.VRPRECECOLIC  := P_ITEMCOLE.VRPRECECOITM;
        V_REC_LOGITEMCOLE.VRPRECOPILIC  := P_ITEMCOLE.VRPRECOITMPI;
        V_REC_LOGITEMCOLE.DSMESSAGERROR := P_DSMESSAGERROR;

        INSERT INTO LOGITEMCOLE
             VALUES V_REC_LOGITEMCOLE;
    
    END INSERT_LOGITEMCOLE;

    PROCEDURE COTACAO_OPL(P_CDFILIAL    IN COLETAPR.CDFILIAL%TYPE,
                          P_NRCOLEORIG  IN COLETAPR.NRCOLEPRECO%TYPE,
                          P_NRCOLEPRECO IN COLETAPR.NRCOLEPRECO%TYPE,
                          P_FORNOL      IN VARCHAR2,
                          P_REAJUSTE    IN NUMBER,
                          P_DTNOVINIVAL IN COLETAPR.DTINIVALCOL%TYPE,
                          P_DTNOVFINVAL IN COLETAPR.DTFINVALCOL%TYPE,
                          P_CDOPERINCL  IN OPERADOR.CDOPERADOR%TYPE) IS
        V_REC_ITEMCOLE S_RECORD_ITEMCOLE;
        C_PARASUPRI    PARASUPRI%ROWTYPE;
        C_FORNECEDORES FORNECEDOR%ROWTYPE;
        V_ERROR        VARCHAR2(4000);
        V_DSIP         VARCHAR2(50);
        V_DSUSUARIO    VARCHAR2(50);
        V_DSRDM        VARCHAR2(50);
        V_DSMETODO     VARCHAR2(250);
        V_DSOPERADOR   VARCHAR2(12);

    BEGIN
        WEBTUNNING(V_DSIP,
                   V_DSUSUARIO,
                   V_DSRDM,
                   V_DSMETODO,
                   V_DSOPERADOR);
        
        SUP_VALIDATIONS.CHECK_PARASUPRI(P_CDFILIAL  => P_CDFILIAL,
                                        P_PARASUPRI => C_PARASUPRI,
                                        P_ERROR     => V_ERROR);
        IF V_ERROR IS NOT NULL THEN
            RETURN;
        END IF;

        FOR C_REGIAO IN (SELECT CDREGIAO, CDFORNOPELOG
                           FROM FILIAL
                          WHERE INSTR(P_FORNOL, CDFORNOPELOG) > 0
                          GROUP BY CDREGIAO, CDFORNOPELOG)
        LOOP                          
            SUP_VALIDATIONS.CHECK_FORNECEDOR(P_CDFORNECED   => C_REGIAO.CDFORNOPELOG,
                                             P_DTPEDIDO     => P_DTNOVFINVAL,
                                             P_PARASUPRI    => C_PARASUPRI,
                                             P_FORNECEDORES => C_FORNECEDORES,
                                             P_MESSAGE      => V_ERROR);
            IF V_ERROR IS NULL THEN
                FOR C_ITENS IN (SELECT ITEMCOLE.CDPRODUTO, RO.CDMARCA, MIN(ITEMCOLE.VRPRECUNIITM) VRPRECUNIITM, 
                                       MIN(ITEMCOLE.VRPRECECOITM) VRPRECECOITM, MIN(ITEMCOLE.VRPERDESITM) VRPERDESITM,
                                       MIN(ITEMCOLE.VRPRECOITMPI) VRPRECOITMPI
                                  FROM ITEMCOLE
                                  JOIN RELACPRODOL RO ON ITEMCOLE.CDPRODUTO = RO.CDPRARVORIG
                                 WHERE ITEMCOLE.CDFILIAL    = P_CDFILIAL
                                   AND ITEMCOLE.NRCOLEPRECO = P_NRCOLEORIG
                                   --AND ITEMCOLE.CDFORNECED  = C_REGIAO.CDFORNOPELOG
                                   AND ITEMCOLE.VRPRECUNIITM > 0
                                 GROUP BY ITEMCOLE.CDPRODUTO, RO.CDMARCA)
                LOOP
                    V_REC_ITEMCOLE.CDFILIAL        := P_CDFILIAL;
                    V_REC_ITEMCOLE.NRCOLEPRECO     := P_NRCOLEPRECO;
                    V_REC_ITEMCOLE.CDREGIAO        := C_REGIAO.CDREGIAO;
                    V_REC_ITEMCOLE.CDFORNECED      := C_REGIAO.CDFORNOPELOG;
                    V_REC_ITEMCOLE.CDPRODUTO       := C_ITENS.CDPRODUTO;
                    V_REC_ITEMCOLE.CDMARCA         := C_ITENS.CDMARCA;
                    
                    V_REC_ITEMCOLE.VRPRECUNIITM    := C_ITENS.VRPRECUNIITM + ((C_ITENS.VRPRECUNIITM * NVL(P_REAJUSTE, 1)) / 100);
                    V_REC_ITEMCOLE.VRPRECECOITM    := C_ITENS.VRPRECECOITM;
                    V_REC_ITEMCOLE.VRPERDESITM     := C_ITENS.VRPERDESITM;
                    V_REC_ITEMCOLE.VRPRECOITMPI    := C_ITENS.VRPRECOITMPI;
                    V_REC_ITEMCOLE.DTINIVALITM     := P_DTNOVINIVAL;
                    V_REC_ITEMCOLE.DTFINVALITM     := P_DTNOVFINVAL;
                    V_REC_ITEMCOLE.NRCOLEORIGIC    := P_NRCOLEORIG;
                    V_REC_ITEMCOLE.CDOPERINCL      := P_CDOPERINCL;
                    V_REC_ITEMCOLE.IDITMLISPREC    := 'N';
                    V_REC_ITEMCOLE.IDPRECALT       := 'N';
                    V_REC_ITEMCOLE.IDITVALDIFLISTA := 'S';
                    V_REC_ITEMCOLE.DTINCLITMCOL    := SYSDATE;

                    INSERT_ITEMCOLE(P_REGISTRO     => V_REC_ITEMCOLE,
                                    P_PARASUPRI    => C_PARASUPRI,
                                    P_FORNECEDORES => C_FORNECEDORES,
                                    P_ERROR        => V_ERROR);

                    IF V_ERROR IS NOT NULL THEN
                        INSERT_LOGITEMCOLE(P_ITEMCOLE      => V_REC_ITEMCOLE,
                                           P_DSENDUSULIC   => V_DSIP,
                                           P_DSNMUSULIC    => V_DSUSUARIO,
                                           P_DSRDMUSULIC   => V_DSRDM,
                                           P_DSMETUSULIC   => V_DSMETODO,
                                           P_CDOPERLIC     => V_DSOPERADOR,
                                           P_IDOPERALIC    => 'I',
                                           P_DSMESSAGERROR => V_ERROR);
                    END IF;
                END LOOP;
            END IF;
        END LOOP;                         
    END COTACAO_OPL;

    PROCEDURE DUPLICA_COTACAO(P_DUPLICA_CORRENTE IN BOOLEAN,
                              P_CDFILIAL         IN COLETAPR.CDFILIAL%TYPE,
                              P_NRCOLEPRECO      IN COLETAPR.NRCOLEPRECO%TYPE,
                              P_CDCOMPRADOR      IN VARCHAR2                  DEFAULT NULL,
                              P_TCDCOMPRADOR     IN VARCHAR2                  DEFAULT NULL,
                              P_CDOPERADOR       IN OPERADOR.CDOPERADOR%TYPE  DEFAULT NULL,
                              P_DTINIVALCOL      IN COLETAPR.DTINIVALCOL%TYPE DEFAULT NULL,
                              P_DTFINVALCOL      IN COLETAPR.DTFINVALCOL%TYPE DEFAULT NULL,
                              P_CDREGIAO         IN VARCHAR2                  DEFAULT NULL,
                              P_DTNOVINIVAL      IN COLETAPR.DTINIVALCOL%TYPE,
                              P_DTNOVFINVAL      IN COLETAPR.DTFINVALCOL%TYPE,
                              P_DTCOLEPREC       IN COLETAPR.DTCOLEPREC%TYPE,
                              P_CDOPERINCL       IN OPERADOR.CDOPERADOR%TYPE,
                              P_REAJUSTE         IN NUMBER,
                              P_CDFORNOL         IN VARCHAR2                  DEFAULT NULL,
                              P_MESSAGE         OUT VARCHAR2) IS                      

        V_QUERY_COTACAO_CORRENTE    VARCHAR2(4000) := 'SELECT *'
                                                   || '  FROM COLETAPR CO'
                                                   || ' WHERE CO.CDFILIAL    = :P_CDFILIAL'
                                                   || '   AND CO.NRCOLEPRECO = :P_NRCOLEPRECO';

        V_QRY_COT_CORRENTE_REGIAO   VARCHAR2(4000) := 'SELECT *'
                                                   || '  FROM COLETAPR CO'
                                                   || ' WHERE CO.CDFILIAL    = :P_CDFILIAL'
                                                   || '   AND CO.NRCOLEPRECO = :P_NRCOLEPRECO'
                                                   || '   AND EXISTS (SELECT ''' || K_N1 || ''''
                                                   || '                 FROM ITEMCOLE'
                                                   || '                WHERE ITEMCOLE.CDFILIAL    = CO.CDFILIAL'
                                                   || '                  AND ITEMCOLE.NRCOLEPRECO = CO.NRCOLEPRECO'
                                                   || '                  AND INSTR(:P_CDREGIAO, ITEMCOLE.CDREGIAO) <> ''' || K_N0 ||''')';

        V_QUERY_COTACAO_FILTRO      VARCHAR2(4000) := 'SELECT *'
                                                   || '  FROM COLETAPR CL'
                                                   || ' WHERE CL.DTINIVALCOL >= :P_DTINIVALCOL'
                                                   || '   AND CL.DTFINVALCOL <= :P_DTFINVALCOL'
                                                   || '   AND ((INSTR(:P_CDCOMPRADOR, CL.CDOPERADOR) <> 0) OR'
                                                   || '        (:P_TCDCOMPRADOR = ''' || K_T || ''' AND EXISTS (SELECT CO.CDOPERADOR'
                                                   || '                                             FROM COMPROPER CE' 
                                                   || '                                             JOIN COMPRADOR CO ON CE.CDCOMPRADOR = CO.CDOPERADOR'
                                                   || '                                             JOIN OPERADOR  OP ON CO.CDOPERADOR  = OP.CDOPERADOR'
                                                   || '                                            WHERE CE.IDCOTCMPOPER = ''' || K_S || ''''
                                                   || '                                              AND CO.CDOPERADOR = CL.CDOPERADOR'
                                                   || '                                              AND CE.CDOPERADOR = :P_CDOPERADOR'
                                                   || '                                            UNION ALL'
                                                   || '                                           SELECT CO.CDOPERADOR'
                                                   || '                                             FROM COMPRADOR CO'
                                                   || '                                             JOIN OPERADOR  OP ON CO.CDOPERADOR = OP.CDOPERADOR'
                                                   || '                                            WHERE CO.CDOPERADOR = CL.CDOPERADOR'
                                                   || '                                              AND CO.CDOPERADOR = :P_CDOPERADOR)))';

        V_QRY_COT_FILTRO_REGIAO     VARCHAR2(4000) := 'SELECT *'
                                                   || '  FROM COLETAPR CL'
                                                   || ' WHERE CL.DTINIVALCOL >= :P_DTINIVALCOL'
                                                   || '   AND CL.DTFINVALCOL <= :P_DTFINVALCOL'
                                                   || '   AND ((INSTR(:P_CDCOMPRADOR, CL.CDOPERADOR) <> 0) OR'
                                                   || '        (:P_TCDCOMPRADOR = ''' || K_T || ''' AND EXISTS (SELECT CO.CDOPERADOR'
                                                   || '                                                           FROM COMPROPER CE'
                                                   || '                                                           JOIN COMPRADOR CO ON CE.CDCOMPRADOR = CO.CDOPERADOR'
                                                   || '                                                           JOIN OPERADOR  OP ON CO.CDOPERADOR  = OP.CDOPERADOR'
                                                   || '                                                          WHERE CE.IDCOTCMPOPER = ''' || K_S ||''''
                                                   || '                                                            AND CO.CDOPERADOR = CL.CDOPERADOR'
                                                   || '                                                            AND CE.CDOPERADOR = :P_CDOPERADOR'
                                                   || '                                                          UNION ALL'
                                                   || '                                                         SELECT CO.CDOPERADOR'
                                                   || '                                                           FROM COMPRADOR CO'
                                                   || '                                                           JOIN OPERADOR  OP ON CO.CDOPERADOR = OP.CDOPERADOR'
                                                   || '                                                          WHERE CO.CDOPERADOR = CL.CDOPERADOR'
                                                   || '                                                            AND CO.CDOPERADOR = :P_CDOPERADOR)))'
                                                   || '  AND EXISTS (SELECT ''' || K_N1 || ''''
                                                   || '                FROM ITEMCOLE'
                                                   || '               WHERE ITEMCOLE.CDFILIAL    = CL.CDFILIAL'
                                                   || '                 AND ITEMCOLE.NRCOLEPRECO = CL.NRCOLEPRECO'
                                                   || '                 AND INSTR(:P_CDREGIAO, ITEMCOLE.CDREGIAO) <> ''' || K_N0 || ''')';

        V_QRY_ITEMCOLE              VARCHAR2(4000) := 'SELECT * '
                                                   || '  FROM ITEMCOLE '
                                                   || ' WHERE CDFILIAL    = :P_CDFILIAL '
                                                   || '   AND NRCOLEPRECO = :P_NRCOLEPRECO ';
        
        TYPE R_COTACAO IS REF CURSOR;
        C_COTACAO      R_COTACAO;
        
        TYPE R_ITEM    IS REF CURSOR;
        C_ITEM         R_ITEM;
        
        C_COLETAPR     COLETAPR%ROWTYPE;
        C_ITEMCOLE     ITEMCOLE%ROWTYPE;
        C_PARASUPRI    PARASUPRI%ROWTYPE;
        C_FORNECEDORES FORNECEDOR%ROWTYPE;
        V_NRCOLEPRECO  VARCHAR2(20); 
        V_REC_COLETAPR S_RECORD_COLETAPR;
        V_REC_ITEMCOLE S_RECORD_ITEMCOLE;
        V_FIRSTROW     BOOLEAN;
        V_SUCCESS      BOOLEAN := TRUE;
        V_ERROR        VARCHAR2(4000);
        V_DSIP         VARCHAR2(50);
        V_DSUSUARIO    VARCHAR2(50);
        V_DSRDM        VARCHAR2(50);
        V_DSMETODO     VARCHAR2(250);
        V_DSOPERADOR   VARCHAR2(12);

    BEGIN
        WEBTUNNING(V_DSIP,
                   V_DSUSUARIO,
                   V_DSRDM,
                   V_DSMETODO,
                   V_DSOPERADOR);

        SUP_VALIDATIONS.CHECK_PARASUPRI(P_CDFILIAL  => P_CDFILIAL,
                                        P_PARASUPRI => C_PARASUPRI,
                                        P_ERROR     => P_MESSAGE);
        IF P_MESSAGE IS NOT NULL THEN
            RETURN;
        END IF;

        IF P_DUPLICA_CORRENTE THEN
            IF P_CDREGIAO IS NULL THEN
                OPEN C_COTACAO FOR V_QUERY_COTACAO_CORRENTE  USING P_CDFILIAL, P_NRCOLEPRECO;
            ELSE
                OPEN C_COTACAO FOR V_QRY_COT_CORRENTE_REGIAO USING P_CDFILIAL, P_NRCOLEPRECO, P_CDREGIAO;
            END IF;
        ELSE
            IF P_CDREGIAO IS NULL THEN
                OPEN C_COTACAO FOR V_QUERY_COTACAO_FILTRO    USING P_DTINIVALCOL, P_DTFINVALCOL, P_CDCOMPRADOR, P_TCDCOMPRADOR, P_CDOPERADOR, P_CDOPERADOR;
            ELSE
                OPEN C_COTACAO FOR V_QRY_COT_FILTRO_REGIAO   USING P_DTINIVALCOL, P_DTFINVALCOL, P_CDCOMPRADOR, P_TCDCOMPRADOR, P_CDOPERADOR, P_CDOPERADOR, P_CDREGIAO;
            END IF;
        END IF;

        IF P_CDREGIAO IS NOT NULL THEN
            V_QRY_ITEMCOLE := V_QRY_ITEMCOLE || ' AND INSTR(P_CDREGIAO, CDREGIAO) <> 0) ';
        END IF;

        LOOP
            FETCH C_COTACAO INTO C_COLETAPR;
            EXIT WHEN C_COTACAO%NOTFOUND;
            V_FIRSTROW := TRUE;
            
            IF NVL(C_PARASUPRI.IDCOTCONCLUI, 'N') = 'S' AND
               NVL(C_PARASUPRI.IDUTISTATCOT, 'N') = 'S' AND 
               NVL(C_COLETAPR.IDSITUCOLETA,  'X') = 'C' THEN
               V_ERROR := K_COTACAO_CONCLUIDA;
            END IF;

            IF P_CDREGIAO IS NOT NULL THEN
                OPEN C_ITEM FOR V_QRY_ITEMCOLE   USING C_COLETAPR.CDFILIAL, C_COLETAPR.NRCOLEPRECO, P_CDREGIAO;
            ELSE
                OPEN C_ITEM FOR V_QRY_ITEMCOLE   USING C_COLETAPR.CDFILIAL, C_COLETAPR.NRCOLEPRECO;
            END IF;

            LOOP
                FETCH C_ITEM INTO C_ITEMCOLE;
                EXIT WHEN C_ITEM%NOTFOUND;

                V_REC_ITEMCOLE := C_ITEMCOLE;
                
                IF V_FIRSTROW THEN
                    V_REC_COLETAPR := C_COLETAPR;
                
                    NOVO_CODIGO('COLETA'||C_COLETAPR.CDFILIAL, V_NRCOLEPRECO);
                    V_REC_COLETAPR.NRCOLEORIGCO := C_COLETAPR.NRCOLEPRECO;
                    V_REC_COLETAPR.NRCOLEPRECO  := SUBSTR(V_NRCOLEPRECO,11,10);
                    V_REC_COLETAPR.DTINIVALCOL  := P_DTNOVINIVAL;
                    V_REC_COLETAPR.DTFINVALCOL  := P_DTNOVFINVAL;
                    V_REC_COLETAPR.DTCOLEPREC   := P_DTCOLEPREC;
                    V_REC_COLETAPR.CDOPERINCL   := P_CDOPERINCL;

                    IF V_ERROR IS NULL THEN
                        INSERT_COLETAPR(P_REGISTRO     => V_REC_COLETAPR,
                                        P_CDFORNECED   => C_ITEMCOLE.CDFORNECED,
                                        P_PARASUPRI    => C_PARASUPRI,
                                        P_FORNECEDORES => C_FORNECEDORES,
                                        P_ERROR        => V_ERROR);
                    END IF;
                    
                    IF V_ERROR IS NOT NULL THEN
                        INSERT_LOGCOLETAPR(P_COLETAPR      => V_REC_COLETAPR,
                                           P_DSENDUSULOG   => V_DSIP,
                                           P_DSNMUSULOG    => V_DSUSUARIO,
                                           P_DSRDMUSULOG   => V_DSRDM,
                                           P_DSMETUSULOG   => V_DSMETODO,
                                           P_CDOPERINCLOG  => V_DSOPERADOR,
                                           P_IDOPERCOTLOG  => 'I',
                                           P_DSMESSAGERROR => V_ERROR);
                    END IF;
                    
                    V_FIRSTROW := FALSE;                    
                END IF;

                IF V_ERROR IS NULL THEN
                    V_REC_ITEMCOLE.VRPRECUNIITM    := V_REC_ITEMCOLE.VRPRECUNIITM + ((V_REC_ITEMCOLE.VRPRECUNIITM * P_REAJUSTE) / 100);
                    V_REC_ITEMCOLE.NRCOLEPRECO     := SUBSTR(V_NRCOLEPRECO,11,10);
                    V_REC_ITEMCOLE.DTINIVALITM     := P_DTNOVINIVAL;
                    V_REC_ITEMCOLE.DTFINVALITM     := P_DTNOVFINVAL;
                    V_REC_ITEMCOLE.NRCOLEORIGIC    := V_REC_COLETAPR.NRCOLEORIGCO;
                    V_REC_ITEMCOLE.CDOPERINCL      := P_CDOPERINCL;
                    V_REC_ITEMCOLE.IDPRECALT       := 'N';
                    V_REC_ITEMCOLE.IDITVALDIFLISTA := 'S';
                    V_REC_ITEMCOLE.DTINCLITMCOL    := SYSDATE;

                    INSERT_ITEMCOLE(P_REGISTRO     => V_REC_ITEMCOLE,
                                    P_PARASUPRI    => C_PARASUPRI,
                                    P_FORNECEDORES => C_FORNECEDORES,
                                    P_ERROR        => V_ERROR);

                    IF V_ERROR IS NOT NULL THEN
                        INSERT_LOGITEMCOLE(P_ITEMCOLE      => V_REC_ITEMCOLE,
                                           P_DSENDUSULIC   => V_DSIP,
                                           P_DSNMUSULIC    => V_DSUSUARIO,
                                           P_DSRDMUSULIC   => V_DSRDM,
                                           P_DSMETUSULIC   => V_DSMETODO,
                                           P_CDOPERLIC     => V_DSOPERADOR,
                                           P_IDOPERALIC    => 'I',
                                           P_DSMESSAGERROR => V_ERROR);
                    END IF;
                END IF;

                V_SUCCESS := V_SUCCESS AND V_ERROR IS NULL;
                V_ERROR   := '';

            END LOOP;            
        END LOOP;
        CLOSE C_COTACAO;
        CLOSE C_ITEM;
        P_MESSAGE := CASE WHEN V_SUCCESS THEN K_SUCCESS ELSE K_ERROR END;
    END DUPLICA_COTACAO;
END SUP_COTACAO;
/
